precision lowp int; 
precision lowp float;

uniform sampler2D sceneMap;
//uniform sampler2D BloomTexture;
uniform sampler2D depthMap;
uniform sampler2D blurMap;
uniform sampler2D vignetteMap;

varying lowp vec4 uvs;
varying lowp vec4 flipuvs;
varying lowp vec3 worldnormal;
varying lowp vec4 vertexcolour;


uniform lowp float dofNear;
uniform lowp float dofFar;

uniform mediump float brightness;
uniform mediump float contrast;
uniform mediump float saturation;
uniform lowp vec3 softLight;
uniform lowp float softLightFactor;
uniform lowp float vignetteFactor;


uniform lowp vec4 fogNear;
uniform lowp vec4 fogFar;
uniform lowp float fogNearValue;
uniform lowp float fogFarValue;


//I <3 Photoshop blend modes
#define BlendLinearDodgef 			BlendAddf
#define BlendLinearBurnf 			BlendSubstractf
#define BlendAddf(base, blend) 		min(base + blend, 1.0)
#define BlendSubstractf(base, blend) 	max(base + blend - 1.0, 0.0)
#define BlendLightenf(base, blend) 		max(blend, base)
#define BlendDarkenf(base, blend) 		min(blend, base)
#define BlendLinearLightf(base, blend) 	(blend < 0.5 ? BlendLinearBurnf(base, (2.0 * blend)) : BlendLinearDodgef(base, (2.0 * (blend - 0.5))))
#define BlendScreenf(base, blend) 		(1.0 - ((1.0 - base) * (1.0 - blend)))
#define BlendOverlayf(base, blend) 	(base < 0.5 ? (2.0 * base * blend) : (1.0 - 2.0 * (1.0 - base) * (1.0 - blend)))
#define BlendSoftLightf(base, blend) 	((blend < 0.5) ? (2.0 * base * blend + base * base * (1.0 - 2.0 * blend)) : (sqrt(base) * (2.0 * blend - 1.0) + 2.0 * base * (1.0 - blend)))
#define BlendColorDodgef(base, blend) 	((blend == 1.0) ? blend : min(base / (1.0 - blend), 1.0))
#define BlendColorBurnf(base, blend) 	((blend == 0.0) ? blend : max((1.0 - ((1.0 - base) / blend)), 0.0))
#define BlendVividLightf(base, blend) 	((blend < 0.5) ? BlendColorBurnf(base, (2.0 * blend)) : BlendColorDodgef(base, (2.0 * (blend - 0.5))))
#define BlendPinLightf(base, blend) 	((blend < 0.5) ? BlendDarkenf(base, (2.0 * blend)) : BlendLightenf(base, (2.0 *(blend - 0.5))))
#define BlendHardMixf(base, blend) 	((BlendVividLightf(base, blend) < 0.5) ? 0.0 : 1.0)
#define BlendReflectf(base, blend) 		((blend == 1.0) ? blend : min(base * base / (1.0 - blend), 1.0))

// Component wise blending
#define Blend(base, blend, funcf) 		vec3(funcf(base.r, blend.r), funcf(base.g, blend.g), funcf(base.b, blend.b))

#define BlendNormal(base, blend) 		(blend)
#define BlendLighten				BlendLightenf
#define BlendDarken				BlendDarkenf
#define BlendMultiply(base, blend) 		(base * blend)
#define BlendAverage(base, blend) 		((base + blend) / 2.0)
#define BlendAdd(base, blend) 		min(base + blend, vec3(1.0))
#define BlendSubstract(base, blend) 	max(base + blend - vec3(1.0), vec3(0.0))
#define BlendDifference(base, blend) 	abs(base - blend)
#define BlendNegation(base, blend) 	(vec3(1.0) - abs(vec3(1.0) - base - blend))
#define BlendExclusion(base, blend) 	(base + blend - 2.0 * base * blend)
#define BlendScreen(base, blend) 		Blend(base, blend, BlendScreenf)
#define BlendOverlay(base, blend) 		Blend(base, blend, BlendOverlayf)
#define BlendSoftLight(base, blend) 	Blend(base, blend, BlendSoftLightf)
#define BlendHardLight(base, blend) 	BlendOverlay(blend, base)
#define BlendColorDodge(base, blend) 	Blend(base, blend, BlendColorDodgef)
#define BlendColorBurn(base, blend) 	Blend(base, blend, BlendColorBurnf)
#define BlendLinearDodge			BlendAdd
#define BlendLinearBurn			BlendSubstract

// Linear Light is another contrast-increasing mode
// If the blend color is darker than midgray, Linear Light darkens the image by decreasing the brightness. If the blend color is lighter than midgray, the result is a brighter image due to increased brightness.
#define BlendLinearLight(base, blend) 	Blend(base, blend, BlendLinearLightf)
#define BlendVividLight(base, blend) 	Blend(base, blend, BlendVividLightf)
#define BlendPinLight(base, blend) 		Blend(base, blend, BlendPinLightf)
#define BlendHardMix(base, blend) 		Blend(base, blend, BlendHardMixf)
#define BlendReflect(base, blend) 		Blend(base, blend, BlendReflectf)
#define BlendGlow(base, blend) 		BlendReflect(blend, base)
#define BlendPhoenix(base, blend) 		(min(base, blend) - max(base, blend) + vec3(1.0))
#define BlendOpacity(base, blend, F, O) 	(F(base, blend) * O + blend * (1.0 - O))

vec3 saturate(vec3 color, float amount)
{
    vec3 gray = vec3(dot(vec3(0.2126,0.7152,0.0722), color));
    return vec3(mix(gray,color, amount));
}

void main() 
{

    vec4 diffuseSample = texture2D(sceneMap, uvs.xy);
    vec4 blurSample = texture2D(blurMap, uvs.xy);
    vec4 vignetteSample = texture2D(vignetteMap, flipuvs.xy);

    float depthSample = texture2D(depthMap, uvs.xy).r;

    



#if defined(USE_DOF)
    //Depth of field  
    lowp float range = dofFar - dofNear;
    lowp float depthValue = depthSample - dofNear;
    depthValue /= range;
    depthValue = clamp(depthValue, 0.0, 1.0);
    vec4 dof = mix(diffuseSample, blurSample, depthValue);
#else
    vec4 dof = diffuseSample;
#endif 


#if defined(USE_VIGNETTE)
    vec3 original = dof.xyz;
    
    dof.xyz = mix(dof.xyz, blurSample.xyz, vignetteSample.a);
    dof.xyz = mix(original,  (dof.xyz*vignetteSample.xyz), vignetteFactor);
#endif 


    
#if defined(USE_FOG)
    //fog
    lowp float fogRange = fogFarValue - fogNearValue;
    lowp float fogValue = depthSample - fogNearValue;
    fogValue /= fogRange;
    fogValue = clamp(fogValue, 0.0, 1.0);
    dof.xyz = mix( mix(dof.xyz, fogNear.xyz, fogNear.w), mix(dof.xyz, fogFar.xyz, fogFar.w), fogValue);
#endif

#if defined(USE_SATURATION)
    //Saturation
    dof.xyz = saturate(dof.xyz, saturation); 

    //contrast
    dof.xyz = (((dof.xyz - 0.5) * contrast) + 0.5);
#endif 

#if defined(USE_SOFTLIGHT)
    //softlight
    dof.xyz = mix(dof.xyz, BlendScreen( dof.xyz, softLight), softLightFactor);
#endif

#if defined(USE_GAMMA_CORRECTION)
    //Gamma
    dof.xyz = pow( dof.xyz, 1.0 / vec3(brightness,brightness,brightness) );
#endif    


    gl_FragColor.xyz = dof.xyz;
    
	gl_FragColor.a = 1.0;
   

}
